@@ -6,13 +6,20 @@ from orangewidget.settings import Setting
 from oasys.widgets import gui as oasysgui, congruence
 from oasys.widgets.exchange import DataExchangeObject
 
-from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
+from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget_dabax import XoppyWidgetDabax
 
 from xoppylib.mlayer import MLayer
 from xoppylib.xoppy_xraylib_util import density
 from xoppylib.xoppy_xraylib_util import Refractive_Index_Re_Extended_NIST, Refractive_Index_Im_Extended_NIST
 
-class OWMlultilayer(XoppyWidget):
+try: import xraylib
+except: print("xraylib not available")
+
+from dabax.dabax_xraylib import DabaxXraylib
+
+from dabax.dabax_files import dabax_f1f2_files, dabax_crosssec_files
+
+class OWMlultilayer(XoppyWidgetDabax):
     name = "Multilayer"
     id = "orange.widgets.datamlayer"
     description = "Multilayer Reflectivity"
@@ -55,10 +62,24 @@ class OWMlultilayer(XoppyWidget):
     def __init__(self):
         super().__init__(show_script_tab=True)
 
+    def dabax_show_f1f2(self):
+        return True
+
+    def dabax_show_crosssec(self):
+        return True
+
+
+
     def build_gui(self):
 
-        box0 = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
-        
+        # box0 = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
+        ###########
+        tabs_setting = oasysgui.tabWidget(self.controlArea)
+        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)
+        box0 = oasysgui.createTabPage(tabs_setting, self.name + " Input Parameters")
+        self.tab_dabax = oasysgui.createTabPage(tabs_setting, "Materials Library")
+        ###########
+
         idx = -1 
 
         #
@@ -388,6 +409,17 @@ class OWMlultilayer(XoppyWidget):
                self.MATERIAL_E, density_E,
                self.MATERIAL_O, density_O, ))
 
+        if self.MATERIAL_CONSTANT_LIBRARY_FLAG == 0:
+            dabax = None
+            material_constants_library_str = "None"
+        else:
+            dabax = DabaxXraylib(file_f1f2=dabax_f1f2_files()[self.DABAX_F1F2_FILE_INDEX],
+                                                      file_CrossSec=dabax_crosssec_files()[self.DABAX_CROSSSEC_FILE_INDEX])
+            material_constants_library_str = 'DabaxXraylib(file_f1f2="%s",file_CrossSec="%s")' % \
+                                             (dabax_f1f2_files()[self.DABAX_F1F2_FILE_INDEX],
+                                              dabax_crosssec_files()[self.DABAX_CROSSSEC_FILE_INDEX])
+            print(dabax.info())
+
         out = MLayer.initialize_from_bilayer_stack(
             material_S=self.MATERIAL_S, density_S=density_S, roughness_S=self.ROUGHNESS_S,  # 2.33
             material_E=self.MATERIAL_E, density_E=density_E, roughness_E=self.ROUGHNESS_E,  # 19.3
@@ -395,6 +427,8 @@ class OWMlultilayer(XoppyWidget):
             bilayer_pairs=self.NLAYERS,
             bilayer_thickness=self.THICKNESS,
             bilayer_gamma=self.GAMMA,
+            use_xraylib_or_dabax=self.MATERIAL_CONSTANT_LIBRARY_FLAG,
+            dabax=dabax,
         )
 
         for key in out.pre_mlayer_dict.keys():
@@ -425,6 +459,49 @@ class OWMlultilayer(XoppyWidget):
             if result == QMessageBox.No:
                 raise Exception("Calculation cancelled.")
 
+        if self.THETA_FLAG == 1 and self.ENERGY_FLAG == 0:   # theta scan
+            myscan = 0
+        elif self.THETA_FLAG == 0 and self.ENERGY_FLAG == 1:  # energy scan
+            myscan = 1
+        elif self.THETA_FLAG == 1 and self.ENERGY_FLAG == 1:  # double scan
+            myscan = 2
+        elif self.THETA_FLAG == 0 and self.ENERGY_FLAG == 0:  # single point
+            myscan = 0
+        print(">>>>>>>>>>>>>>>>>>>>>> myscan: ", myscan)
+        #
+        # script
+        #
+        dict_parameters = {
+            "material_S":    self.MATERIAL_S,
+            "density_S":     density_S,
+            "roughness_S":   self.ROUGHNESS_S,
+            "material_E":    self.MATERIAL_E,
+            "density_E":     density_E,
+            "roughness_E":   self.ROUGHNESS_E,
+            "material_O":    self.MATERIAL_O,
+            "density_O":     density_O,
+            "roughness_O":   self.ROUGHNESS_O,
+            "bilayer_pairs": self.NLAYERS,
+            "bilayer_thickness": self.THICKNESS,
+            "bilayer_gamma":     self.GAMMA,
+            "energyN":           energyN,
+            "energy1":           self.ENERGY,
+            "energy2":           self.ENERGY_END,
+            "thetaN":            thetaN,
+            "theta1":            self.THETA,
+            "theta2":            self.THETA_END,
+            "myscan":            myscan,
+            "h5file":            h5file,
+            "use_xraylib_or_dabax" : self.MATERIAL_CONSTANT_LIBRARY_FLAG,
+            "dabax_str"            : material_constants_library_str,
+            }
+        # write python script
+        self.xoppy_script.set_code(self.script_template().format_map(dict_parameters))
+
+
+        #
+        # calculation
+        #
         rs, rp, e, t = out.scan(h5file=h5file,
                                 energyN=energyN, energy1=self.ENERGY, energy2=self.ENERGY_END,
                                 thetaN=thetaN, theta1=self.THETA, theta2=self.THETA_END)
@@ -436,86 +513,55 @@ class OWMlultilayer(XoppyWidget):
 
         if self.THETA_FLAG == 1 and self.ENERGY_FLAG == 0:   # theta scan
             out = numpy.zeros((t.size,3))
-
             out[:, 0] = t
             out[:, 1] = rs[0]**2
             out[:, 2] = rp[0]**2
-
             out_dict["data"] = out
-            myscan = 0
-
         elif self.THETA_FLAG == 0 and self.ENERGY_FLAG == 1:  # energy scan
             out = numpy.zeros((e.size,3))
-
             out[:, 0] = e
             out[:, 1] = rs[:, 0]**2
             out[:, 2] = rp[:, 0]**2
-
             out_dict["data"] = out
-            myscan = 1
-
         elif self.THETA_FLAG == 1 and self.ENERGY_FLAG == 1:  # double scan
-
             out_dict["data2D_rs"] = rs**2
             out_dict["data2D_rp"] = rs**2
             out_dict["dataX"] = e
             out_dict["dataY"] = t
-            myscan = 2
-
         elif self.THETA_FLAG == 0 and self.ENERGY_FLAG == 0:  # single point
             out = numpy.zeros((t.size,3))
             out[:, 0] = t
             out[:, 1] = rs[0]**2
             out[:, 2] = rp[0]**2
-
             out_dict["data"] = out
-            myscan = 0
-
 
         calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
 
         try:
             calculated_data.add_content("xoppy_data", out_dict["data"])
             calculated_data.add_content("plot_x_col", 0)
-            calculated_data.add_content("plot_y_col", 1)
+            calculated_data.add_content("plot_y_col", -1)
+            calculated_data.add_content("myscan", myscan)
         except:
             pass
 
         try:
             calculated_data.add_content("data2D_rs", out_dict["data2D_rs"])
             calculated_data.add_content("data2D_rp", out_dict["data2D_rp"])
-            calculated_data.add_content("dataX", out_dict["dataX"])
-            calculated_data.add_content("dataY", out_dict["dataY"])
+            calculated_data.add_content("dataX",     out_dict["dataX"])
+            calculated_data.add_content("dataY",     out_dict["dataY"])
         except:
             pass
 
-        # script
-
-        dict_parameters = {
-            "material_S":    self.MATERIAL_S,
-            "density_S":     density_S, \
-            "roughness_S":   self.ROUGHNESS_S, \
-            "material_E":    self.MATERIAL_E, \
-            "density_E":     density_E, \
-            "roughness_E":   self.ROUGHNESS_E, \
-            "material_O":    self.MATERIAL_O, \
-            "density_O":     density_O, \
-            "roughness_O":   self.ROUGHNESS_O, \
-            "bilayer_pairs": self.NLAYERS, \
-            "bilayer_thickness": self.THICKNESS, \
-            "bilayer_gamma":     self.GAMMA, \
-            "energyN":           energyN, \
-            "energy1":           self.ENERGY, \
-            "energy2":           self.ENERGY_END, \
-            "thetaN":            thetaN, \
-            "theta1":            self.THETA,\
-            "theta2":            self.THETA_END,
-            "myscan":            myscan, \
-            "h5file":            h5file, \
-            }
-
-        # write python script
-        self.xoppy_script.set_code(self.script_template().format_map(dict_parameters))
+        try:
+            calculated_data.add_content("THETA_N",    self.THETA_N)
+            calculated_data.add_content("THETA",      self.THETA)
+            calculated_data.add_content("THETA_END",  self.THETA_END)
+            calculated_data.add_content("ENERGY_N",   self.ENERGY_N)
+            calculated_data.add_content("ENERGY",     self.ENERGY)
+            calculated_data.add_content("ENERGY_END", self.ENERGY_END)
+        except:
+            pass
 
         return calculated_data
 
@@ -523,6 +569,9 @@ class OWMlultilayer(XoppyWidget):
 
     def script_template(self):
         return """
+try: import xraylib
+except: print("xraylib not available")
+from dabax.dabax_xraylib import DabaxXraylib
 from xoppylib.mlayer import MLayer
 
 out = MLayer.initialize_from_bilayer_stack(
@@ -532,6 +581,8 @@ out = MLayer.initialize_from_bilayer_stack(
     bilayer_pairs={bilayer_pairs},
     bilayer_thickness={bilayer_thickness},
     bilayer_gamma={bilayer_gamma},
+    use_xraylib_or_dabax = {use_xraylib_or_dabax},
+    dabax={dabax_str},
 )
 
 for key in out.pre_mlayer_dict.keys():
@@ -544,15 +595,16 @@ rs, rp, e, t = out.scan(h5file="{h5file}",
 #
 # plot (example)
 #
-myscan = {myscan}
-from srxraylib.plot.gol import plot,plot_image
-
-if myscan == 0: # angle scan 
-    plot(t, rs[0]**2, xtitle="angle [deg]", ytitle="Reflectivity-s", title="")
-elif myscan == 1: # energy scan 
-    plot(e,rs[:,0]**2,xtitle="Photon energy [eV]",ytitle="Reflectivity-s",title="")
-elif myscan == 2: # double scan 
-    plot_image(rs**2,e,t,xtitle="Photon energy [eV]",ytitle="Grazing angle [deg]",title="Reflectivity-s",aspect="auto")
+if True:
+    myscan = {myscan}
+    from srxraylib.plot.gol import plot,plot_image
+    
+    if myscan == 0: # angle scan 
+        plot(t, rs[0]**2, xtitle="angle [deg]", ytitle="Reflectivity-s", title="")
+    elif myscan == 1: # energy scan 
+        plot(e,rs[:,0]**2,xtitle="Photon energy [eV]",ytitle="Reflectivity-s",title="")
+    elif myscan == 2: # double scan 
+        plot_image(rs**2,e,t,xtitle="Photon energy [eV]",ytitle="Grazing angle [deg]",title="Reflectivity-s",aspect="auto")
                
 """
 
