@@ -13,8 +13,9 @@ from orangewidget.settings import Setting
 from oasys.widgets import gui as oasysgui, congruence
 from oasys.widgets.exchange import DataExchangeObject
 from oasys.widgets.gui import ConfirmDialog
+from oasys.util.oasys_objects import OasysSurfaceData
 
-from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
+from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget_dabax import XoppyWidgetDabax
 
 from xoppylib.power.power3d import integral_2d, integral_3d, info_total_power
 from xoppylib.power.power3d import calculate_component_absorbance_and_transmittance, apply_transmittance_to_incident_beam
@@ -27,7 +28,14 @@ from syned.beamline.optical_elements.mirrors.mirror import Mirror
 from syned.beamline.beamline import Beamline
 from syned.beamline.shape import Rectangle
 
-class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
+try: import xraylib
+except: print("xraylib not available")
+
+from dabax.dabax_xraylib import DabaxXraylib
+from dabax.dabax_files import dabax_f1f2_files, dabax_crosssec_files
+
+
+class OWpower3Dcomponent(XoppyWidgetDabax, WidgetDecorator):
     name = "Power3Dcomponent"
     id = "orange.widgets.datapower3D"
     description = "Power (vs Energy and spatial coordinates) Absorbed and Transmitted or Reflected by Optical Elements"
@@ -36,21 +44,28 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
     category = ""
     keywords = ["xoppy", "Undulator Radiation", "power3Dcomponent"]
 
-    inputs = [{"name": "ExchangeData",
+    inputs = [{"name": "xoppy_data",
                "type": DataExchangeObject,
                "handler": "acceptExchangeData" },
-              WidgetDecorator.syned_input_data()[0]]
+              {"name": "ExchangeData",
+               "type": DataExchangeObject,
+               "handler": "acceptExchangeData" },
+              WidgetDecorator.syned_input_data()[0],
+              ("Surface Data 1", OasysSurfaceData, "set_input_surface_data_front"),
+              ("Surface Data 2", OasysSurfaceData, "set_input_surface_data_back")]
+
 
     INPUT_BEAM_FROM = Setting(0)
     INPUT_BEAM_FILE = Setting("undulator_radiation.h5")
 
     EL1_FOR = Setting("Be")
-    EL1_FLAG = Setting(0)  # 0=Filter 1=Mirror 2 = Aperture 3 magnifier
+    EL1_FLAG = Setting(0)  # 0=Filter 1=Mirror 2=Aperture 3=magnifier, 4=Screen rotation  5=Thin object  6=Multilayer 7=External file
     EL1_THI = Setting(0.5)
     EL1_ANG = Setting(3.0)
     EL1_DEF = Setting(1) # deflection 0=H 1=V
     EL1_ROU = Setting(0.0)
     EL1_DEN = Setting("?")
+    EL1_GAPSHAPE = Setting(0)
     EL1_HGAP = Setting(1000.0)
     EL1_VGAP = Setting(1000.0)
     EL1_HGAPCENTER = Setting(0.0)
@@ -59,6 +74,12 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
     EL1_VMAG = Setting(1.0)
     EL1_HROT = Setting(0.0)
     EL1_VROT = Setting(0.0)
+    thin_object_file = Setting('<none>')
+    thin_object_thickness_outside_file_area = Setting(0.0)
+    thin_object_back_profile_flag = Setting(0)
+    thin_object_back_profile_file = Setting('<none>')
+    multilayer_file = Setting('<none>')
+    external_reflectivity_file = Setting('<none>')
 
     PLOT_SETS = Setting(1)
 
@@ -71,9 +92,17 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
     INTERPOLATION_FACTOR_H = Setting(1.0)
     INTERPOLATION_FACTOR_V = Setting(1.0)
 
+
+
     def __init__(self):
         super().__init__(show_script_tab=True)
 
+    def dabax_show_f1f2(self):
+        return True
+
+    def dabax_show_crosssec(self):
+        return True
+
     def build_gui(self):
 
         self.leftWidgetPart.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding))
@@ -85,6 +114,7 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
         tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)
         tab_1 = oasysgui.createTabPage(tabs_setting, "Input Parameters")
         tab_2 = oasysgui.createTabPage(tabs_setting, "Send Settings")
+        self.tab_dabax = oasysgui.createTabPage(tabs_setting, "Materials Library")
         ###########
 
         box = oasysgui.widgetBox(tab_1, self.name + " Input Parameters", orientation="vertical",width=self.CONTROL_AREA_WIDTH-10)
@@ -117,7 +147,7 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
         box1 = gui.widgetBox(box11)
         gui.comboBox(box1, self, "EL1_FLAG",
                     label=self.unitLabels()[idx], addSpace=False,
-                    items=['Filter', 'Mirror','Aperture','Magnifier','Screen Rotation'],
+                    items=['Filter', 'Mirror','Aperture','Magnifier','Screen Rotation',"Thin object filter","Multilayer","Reflectivity from file"],
                     valueType=int, orientation="horizontal", callback=self.set_EL_FLAG, labelWidth=250)
         self.show_at(self.unitFlags()[idx], box1)
 
@@ -130,10 +160,12 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
         self.show_at(self.unitFlags()[idx], box1)
 
 
-        list_w = ["EL1_THI", "EL1_ANG", "EL1_DEF", "EL1_ROU", "EL1_DEN",
-                  "EL1_HGAP", "EL1_VGAP", "EL1_HGAPCENTER", "EL1_VGAPCENTER",
+        list_w = ["EL1_THI", "EL1_ANG", "EL1_DEF", "EL1_ROU", "EL1_DEN", "multilayer_file", "external_reflectivity_file",
+                  "EL1_GAPSHAPE", "EL1_HGAP", "EL1_VGAP", "EL1_HGAPCENTER", "EL1_VGAPCENTER",
                   "EL1_HMAG", "EL1_VMAG",
-                  "EL1_HROT", "EL1_VROT"]
+                  "EL1_HROT", "EL1_VROT",
+                  "thin_object_file", "thin_object_thickness_outside_file_area", "thin_object_back_profile_flag", "thin_object_back_profile_file",
+                  ]
 
         for el in list_w:
             idx += 1
@@ -144,10 +176,30 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
                              items=['Horizontal',
                                     'Vertical'],
                              valueType=int, orientation="horizontal", labelWidth=250)
-            elif el == "EL1_DEN":
+            elif el == "EL1_GAPSHAPE":
+                gui.comboBox(box1, self, el,
+                             label=self.unitLabels()[idx], addSpace=False,
+                             items=['Rectangle',
+                                    'Ellipse'],
+                             valueType=int, orientation="horizontal", labelWidth=250)
+            elif el == "thin_object_back_profile_flag":
+                gui.comboBox(box1, self, el,
+                             label=self.unitLabels()[idx], addSpace=False,
+                             items=['zero',
+                                    'from h5 file'],
+                             valueType=int, orientation="horizontal", labelWidth=250)
+            elif el in ["EL1_DEN", "thin_object_file", "thin_object_back_profile_file"]:
                 oasysgui.lineEdit(box1, self, el,
                                   label=self.unitLabels()[idx], addSpace=False,
                                   valueType=str, orientation="horizontal", labelWidth=250)
+            elif el in ["multilayer_file"]:
+                oasysgui.lineEdit(box1, self, el,
+                                  label=self.unitLabels()[idx], addSpace=False,
+                                  valueType=str, orientation="horizontal", labelWidth=200)
+            elif el in ["external_reflectivity_file"]:
+                oasysgui.lineEdit(box1, self, el,
+                                  label=self.unitLabels()[idx], addSpace=False,
+                                  valueType=str, orientation="horizontal", labelWidth=200)
             else:
                 oasysgui.lineEdit(box1, self, el,
                                   label=self.unitLabels()[idx], addSpace=False,
@@ -271,18 +323,27 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
         labels.append('optical element is:')
         labels.append('formula: ')
         labels.append('Filter thick[mm]')
-        labels.append('Mirror angle[mrad]')
-        labels.append('Mirror deflection')
+        labels.append('Grazing angle[mrad]')
+        labels.append('Component deflection')
         labels.append('Roughness[A]')
         labels.append('Density [g/cm^3]')
+        labels.append('File from xoppy/Multilayer')
+        labels.append('File with reflectivity vs E')
+        labels.append('Aperture shape')
         labels.append('H Size/Gap [mm]')
         labels.append('V Size/Gap [mm]')
         labels.append('H Center/Gap [mm]')
         labels.append('V Center/Gap [mm]')
         labels.append('H Magnification')
         labels.append('V Magnification')
-        labels.append('Rotation angle around V axis [deg]')
         labels.append('Rotation angle around H axis [deg]')
+        labels.append('Rotation angle around V axis [deg]')
+        labels.append('File with FRONT profile z(x,y)')
+        labels.append('Front profile z outside file definition')
+        labels.append('Back profile z(x,y)')
+        labels.append('File with BACK profile z(x,y)')
+
+
 
         labels.append("Plot")
         labels.append("Write input beam")
@@ -301,20 +362,27 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
         flags.append('True')  # input from
         flags.append('self.INPUT_BEAM_FROM  ==  1')  # input file
         flags.append('True')                   # kind
-        flags.append('self.EL1_FLAG  <=  1')   # formula
+        flags.append('self.EL1_FLAG  in  (0, 1, 5)')   # formula
         flags.append('self.EL1_FLAG  ==  0')   # thickness
-        flags.append('self.EL1_FLAG  ==  1')   # angle
-        flags.append('self.EL1_FLAG  ==  1')   # mirror deflection
+        flags.append('self.EL1_FLAG  in  (1, 6)')   # angle
+        flags.append('self.EL1_FLAG  in  (1, 6)')   # mirror deflection
         flags.append('self.EL1_FLAG  ==  1')   # roughness
-        flags.append('self.EL1_FLAG  <=  1')   # density
-        flags.append('self.EL1_FLAG  <=  2')   # gap
-        flags.append('self.EL1_FLAG  <=  2')   # gap
-        flags.append('self.EL1_FLAG  <=  2')   # gap center
-        flags.append('self.EL1_FLAG  <=  2')   # gap center
+        flags.append('self.EL1_FLAG  in  (0, 1, 5)')   # density
+        flags.append('self.EL1_FLAG  == 6')  # multilayer file
+        flags.append('self.EL1_FLAG  == 7')   # file with external reflectivity
+        flags.append('self.EL1_FLAG  in (0, 1, 2, 6, 7)')  # gap shape
+        flags.append('self.EL1_FLAG  in (0, 1, 2, 6, 7)')   # gap
+        flags.append('self.EL1_FLAG  in (0, 1, 2, 6, 7)')   # gap
+        flags.append('self.EL1_FLAG  in (0, 1, 2, 6, 7)')   # gap center
+        flags.append('self.EL1_FLAG  in (0, 1, 2, 6, 7)')   # gap center
         flags.append('self.EL1_FLAG  ==  3')   # magnification
         flags.append('self.EL1_FLAG  ==  3')   # magnification
-        flags.append('self.EL1_FLAG  in (0, 4)')   # rotation
-        flags.append('self.EL1_FLAG  in (0, 4)')   # rotation
+        flags.append('self.EL1_FLAG  in (0, 4, 7)')   # rotation
+        flags.append('self.EL1_FLAG  in (0, 4, 7)')   # rotation
+        flags.append('self.EL1_FLAG  == 5')   # thin object thickness
+        flags.append('self.EL1_FLAG  == 5')   # thin object thickness
+        flags.append('self.EL1_FLAG  == 5')   # thin object back profile flag
+        flags.append('self.EL1_FLAG  == 5 and self.thin_object_back_profile_flag == 1')   # thin object back profile file
         flags.append("True")  # plot
         flags.append("True")
         flags.append('self.FILE_INPUT_FLAG >= 1')
@@ -329,20 +397,92 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
     def get_help_name(self):
         return 'power3dcomponent'
 
+    def set_input_surface_data_front(self, surface_data):
+        if isinstance(surface_data, OasysSurfaceData):
+            self.EL1_FLAG = 5
+            self.thin_object_file = surface_data.surface_data_file
+            self.thin_object_thickness_outside_file_area = 0.0
+        else:
+            raise Exception("Wrong surface_data")
+
+    def set_input_surface_data_back(self, surface_data):
+        if isinstance(surface_data, OasysSurfaceData):
+            self.EL1_FLAG = 5
+            self.thin_object_back_profile_flag= 1
+            self.thin_object_back_profile_file= surface_data.surface_data_file
+        else:
+            raise Exception("Wrong surface_data")
+
     def acceptExchangeData(self, exchangeData):
         try:
             if not exchangeData is None:
-
                 if exchangeData.get_program_name() not in ["XOPPY"]:
-                        raise Exception("Exchange data must be XOPPY data")
+                    raise Exception("Exchange data must be XOPPY data")
 
-                if exchangeData.get_widget_name() not in ["UNDULATOR_RADIATION", "WIGGLER_RADIATION", "POWER3DCOMPONENT"]:
-                        raise Exception("Xoppy Input widget not recognized: %s" % exchangeData.get_widget_name())
+                name = exchangeData.get_widget_name()
+                if name in ["UNDULATOR_RADIATION", "WIGGLER_RADIATION", "POWER3DCOMPONENT"]:
+                    try:
+                        self.input_beam = exchangeData
+                        self.output_beam = None
+                        self.process_showers()
+                        self.compute()
+                    except:
+                        raise Exception("Error loading Input data from Xoppy Input widget: %s" % exchangeData.get_widget_name())
+                elif name == "MULTILAYER":
+                    myscan = -1
+                    if exchangeData.has_content_key("myscan"): myscan = exchangeData.get_content("myscan")
+                    if myscan == 1: # only energy scan allowed
+                        x_index = exchangeData.get_content("plot_x_col")
+                        y_index = exchangeData.get_content("plot_y_col")
+                        reflectivity = exchangeData.get_content("xoppy_data")
+                        reflectivity[numpy.where(numpy.isnan(reflectivity))] = 0
+
+                        external_reflectivity_file = "xoppy_reflectivity_" + str(id(self)) + ".dat"
+                        file = open(external_reflectivity_file, "w")
+                        for index in range(0, reflectivity.shape[0]):
+                            file.write(str(reflectivity[index, x_index]) + " " + str(reflectivity[index, y_index]) + "\n")
+                        file.close()
+
+                        self.external_reflectivity_file = external_reflectivity_file
+                        self.EL1_FLAG = 7
+
+                        self.EL1_HROT = numpy.round(90.0 - (exchangeData.get_content("THETA")), 4)
+                        self.EL1_VROT = 0.0
+
+                    else:
+                        raise Exception("Only energy scans allowed.")
+
+                elif name == "XCRYSTAL":
+                    if exchangeData.get_content("scan_type") == 3:
+                        x_index = 0 # energy
+                        y_index = -1 # s-pol
+
+                        reflectivity = exchangeData.get_content("xoppy_data")
+
+                        external_reflectivity_file = "xoppy_reflectivity_" + str(id(self)) + ".dat"
+                        file = open(external_reflectivity_file, "w")
+
+                        for index in range(0, reflectivity.shape[0]):
+                            file.write(
+                                str(reflectivity[index, x_index]) + " " + str(reflectivity[index, y_index]) + "\n")
+                        file.close()
+
+                        self.external_reflectivity_file = external_reflectivity_file
+
+
+                        self.EL1_FLAG = 7
+
+                        self.EL1_HROT = numpy.round(90.0 - \
+                                                    (exchangeData.get_content("bragg_angle") + \
+                                                     exchangeData.get_content("asymmetry_angle")), 4)
+                        self.EL1_VROT = 0.0
+
+                    else:
+                        raise Exception("Only Energy Scan are accepted from CRYSTAL")
+
+                else:
+                    raise Exception("Cannot recover correct data from widget: %s" % name)
 
-                self.input_beam = exchangeData
-                self.output_beam = None
-                self.process_showers()
-                self.compute()
         except Exception as exception:
             QMessageBox.critical(self, "Error", str(exception), QMessageBox.Ok)
 
@@ -380,8 +520,8 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
         elif self.EL1_FLAG == 1: # mirror
             self.EL1_ANG = congruence.checkStrictlyPositiveNumber(self.EL1_ANG, "OE mirror angle")
             self.EL1_ROU = congruence.checkPositiveNumber(self.EL1_ROU, "OE mirror roughness")
-            self.EL1_HROT = congruence.checkNumber(self.EL1_HROT, "OE rotation H")
-            self.EL1_VROT = congruence.checkNumber(self.EL1_VROT, "OE rotation V")
+            self.EL1_HROT = congruence.checkNumber(self.EL1_HROT, "OE rotation around H axis")
+            self.EL1_VROT = congruence.checkNumber(self.EL1_VROT, "OE rotation around V axis")
             self.EL1_HGAP = congruence.checkStrictlyPositiveNumber(self.EL1_HGAP, "OE H gap")
             self.EL1_VGAP = congruence.checkPositiveNumber(self.EL1_VGAP, "OE V Gap")
             self.EL1_HGAPCENTER = congruence.checkNumber(self.EL1_HGAPCENTER, "OE H gap Center")
@@ -395,8 +535,10 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
             self.EL1_HMAG = congruence.checkStrictlyPositiveNumber(self.EL1_HMAG, "OE H magnification")
             self.EL1_VMAG = congruence.checkPositiveNumber(self.EL1_VMAG, "OE V magnification")
         elif self.EL1_FLAG == 4: # rotation
-            self.EL1_HROT = congruence.checkNumber(self.EL1_HROT, "OE rotation H")
-            self.EL1_VROT = congruence.checkNumber(self.EL1_VROT, "OE rotation V")
+            self.EL1_HROT = congruence.checkNumber(self.EL1_HROT, "OE rotation around H axis")
+            self.EL1_VROT = congruence.checkNumber(self.EL1_VROT, "OE rotation around V axis")
+        elif self.EL1_FLAG == 4: # thin object filter
+            self.thin_object_thickness_outside_file_area = congruence.checkNumber(self.thin_object_thickness_outside_file_area, "Thin object thickness outside file")
 
         if self.FILE_DUMP == 1:
             if (os.path.splitext(self.FILE_NAME))[-1] not in [".h5",".H5",".hdf5",".HDF5"]:
@@ -434,24 +576,150 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
                                    h5_parameters=None,
                                    )
 
+        # defaults
+        gapshape = 0
+        hgap = 1000.0
+        vgap = 1000.0
+        hgapcenter = 0.0
+        vgapcenter = 0.0
+        hmag = 1.0
+        vmag = 1.0
+        hrot = 0.0
+        vrot = 0.0
+        thin_object_file = ''
+        thin_object_thickness_outside_file_area = 0.0
+        thin_object_back_profile_flag = 0
+        thin_object_back_profile_file = ''
+        multilayer_file = ''
+        external_reflectivity_file = ''
+
+        if self.EL1_FLAG in [0,1,2,5,6,7]: #  using slit
+            hgap = self.EL1_HGAP
+            vgap = self.EL1_VGAP
+            hgapcenter = self.EL1_HGAPCENTER
+            vgapcenter = self.EL1_VGAPCENTER
+            gapshape = self.EL1_GAPSHAPE
+
+        if self.EL1_FLAG == 3:
+            hmag = self.EL1_HMAG
+            vmag = self.EL1_VMAG
+
+        if self.EL1_FLAG in [0,4,7]: #  using rotation
+            hrot = self.EL1_HROT
+            vrot = self.EL1_VROT
+
+        if self.EL1_FLAG == 5:
+            thin_object_file = self.thin_object_file
+            thin_object_thickness_outside_file_area = self.thin_object_thickness_outside_file_area
+            thin_object_back_profile_flag = self.thin_object_back_profile_flag
+            thin_object_back_profile_file = self.thin_object_back_profile_file
+
+        if self.EL1_FLAG == 6:
+            multilayer_file = self.multilayer_file
+
+        if self.EL1_FLAG == 7:
+            external_reflectivity_file = self.external_reflectivity_file
+
+        #
+        # dabax stuff
+        #
+        if self.MATERIAL_CONSTANT_LIBRARY_FLAG == 0:
+            material_constants_library = xraylib
+            material_constants_library_str = "xraylib"
+        else:
+            material_constants_library = DabaxXraylib(file_f1f2=dabax_f1f2_files()[self.DABAX_F1F2_FILE_INDEX],
+                                                      file_CrossSec=dabax_crosssec_files()[self.DABAX_CROSSSEC_FILE_INDEX])
+            material_constants_library_str = 'DabaxXraylib(file_f1f2="%s",file_CrossSec="%s")' % \
+                                             (dabax_f1f2_files()[self.DABAX_F1F2_FILE_INDEX],
+                                              dabax_crosssec_files()[self.DABAX_CROSSSEC_FILE_INDEX])
+            print(material_constants_library.info())
+
+        #
+        # write python script
+        #
+        if isinstance(self.EL1_DEN, str):
+            dens = "'"+self.EL1_DEN+"'"
+        else:
+            dens = "%g" % self.EL1_DEN
+
+        dict_parameters = {
+                "emin" :    e0[0],
+                "emax" :    e0[-1],
+                "epoints" : e0.size,
+                "hmin" :    h0[0],
+                "hmax" :    h0[-1],
+                "hpoints" : h0.size,
+                "vmin" :    v0[0],
+                "vmax" :    v0[-1],
+                "vpoints" : v0.size,
+                "EL1_FOR" : "'"+self.EL1_FOR+"'",
+                "EL1_THI" : self.EL1_THI,
+                "EL1_ANG" : self.EL1_ANG,
+                "EL1_DEF" : self.EL1_DEF,
+                "EL1_DEN" : dens,
+                "EL1_ROU" : self.EL1_ROU,
+                "EL1_FLAG" : self.EL1_FLAG,
+                "hgap" : hgap,
+                "vgap" : vgap,
+                "hgapcenter" : hgapcenter,
+                "vgapcenter" : vgapcenter,
+                "hmag" : hmag,
+                "vmag" : vmag,
+                "hrot" : hrot,
+                "vrot" : vrot,
+                "FILE_INPUT_NAME": self.FILE_INPUT_NAME,
+                "INTERPOLATION_FLAG" : self.INTERPOLATION_FLAG,
+                "INTERPOLATION_FACTOR_H" : self.INTERPOLATION_FACTOR_H,
+                "INTERPOLATION_FACTOR_V" : self.INTERPOLATION_FACTOR_V,
+                "EL1_SLIT_CROP" : self.EL1_SLIT_CROP,
+                "thin_object_file" : thin_object_file,
+                "thin_object_thickness_outside_file_area" : thin_object_thickness_outside_file_area,
+                "thin_object_back_profile_flag": thin_object_back_profile_flag,
+                "thin_object_back_profile_file": thin_object_back_profile_file,
+                "multilayer_file":  multilayer_file,
+                "external_reflectivity_file": external_reflectivity_file,
+                "material_constants_library": material_constants_library_str,
+            }
+
+        if self.input_beam is not None:
+            try:
+                script_previous = self.input_beam.get_content("xoppy_script")
+            except:
+                script_previous = '#\n# >> MISSING SCRIPT TO CREATE (energy, horizontal, vertical, flux3D) <<\n#\n'
+        script_element = self.script_template().format_map(dict_parameters)
+        script = script_previous + script_element
+        self.xoppy_script.set_code(script)
+
+
+        #
+        #calculation
+        #
         transmittance, absorbance, E, H, V, txt = calculate_component_absorbance_and_transmittance(
-                                                              e0, h0, v0,
-                                                              substance=self.EL1_FOR,
-                                                              thick=self.EL1_THI,
-                                                              angle=self.EL1_ANG,
-                                                              defection=self.EL1_DEF,
-                                                              dens=self.EL1_DEN,
-                                                              roughness=self.EL1_ROU,
-                                                              flags=self.EL1_FLAG,
-                                                              hgap=self.EL1_HGAP,
-                                                              vgap=self.EL1_VGAP,
-                                                              hgapcenter=self.EL1_HGAPCENTER,
-                                                              vgapcenter=self.EL1_VGAPCENTER,
-                                                              hmag=self.EL1_HMAG,
-                                                              vmag=self.EL1_VMAG,
-                                                              hrot=self.EL1_HROT,
-                                                              vrot=self.EL1_VROT,
-                                                              )
+                                  e0, h0, v0,
+                                  substance=self.EL1_FOR,
+                                  thick=self.EL1_THI,
+                                  angle=self.EL1_ANG,
+                                  defection=self.EL1_DEF,
+                                  dens=self.EL1_DEN,
+                                  roughness=self.EL1_ROU,
+                                  flags=self.EL1_FLAG,
+                                  gapshape=gapshape,
+                                  hgap=hgap,
+                                  vgap=vgap,
+                                  hgapcenter=hgapcenter,
+                                  vgapcenter=vgapcenter,
+                                  hmag=hmag,
+                                  vmag=vmag,
+                                  hrot=hrot,
+                                  vrot=vrot,
+                                  thin_object_file=thin_object_file,
+                                  thin_object_thickness_outside_file_area=thin_object_thickness_outside_file_area,
+                                  thin_object_back_profile_flag=thin_object_back_profile_flag,
+                                  thin_object_back_profile_file=thin_object_back_profile_file,
+                                  multilayer_file=multilayer_file,
+                                  external_reflectivity_file=external_reflectivity_file,
+                                  material_constants_library=material_constants_library,
+                                  )
 
         txt += info_total_power(p0, e0, v0, h0, transmittance, absorbance, EL1_FLAG=self.EL1_FLAG)
         print(txt)
@@ -462,7 +730,7 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
             pass
         elif self.FILE_DUMP == 1:
             write_h5_file(calculated_data, self.input_beam.get_content("xoppy_data"), filename=self.FILE_NAME,
-                          EL1_FLAG=self.EL1_FLAG, EL1_HMAG=self.EL1_HMAG, EL1_VMAG=self.EL1_VMAG)
+                          EL1_FLAG=self.EL1_FLAG, EL1_HMAG=hmag, EL1_VMAG=vmag)
         elif self.FILE_DUMP == 2:
             write_txt_file(calculated_data, self.input_beam.get_content("xoppy_data"),
                            filename=self.FILE_NAME, method="3columns")
@@ -471,57 +739,6 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
                            filename=self.FILE_NAME, method="matrix")
 
 
-        if isinstance(self.EL1_DEN, str):
-            dens = "'"+self.EL1_DEN+"'"
-        else:
-            dens = "%g" % self.EL1_DEN
-
-        # write python script
-        dict_parameters = {
-                "emin" : E[0],
-                "emax" : E[-1],
-                "epoints" : E.size,
-                "hmin" : H[0],
-                "hmax" : H[-1],
-                "hpoints" : H.size,
-                "vmin" : V[0],
-                "vmax" : V[-1],
-                "vpoints" : V.size,
-                "EL1_FOR" : "'"+self.EL1_FOR+"'",
-                "EL1_THI" : self.EL1_THI,
-                "EL1_ANG" : self.EL1_ANG,
-                "EL1_DEF" : self.EL1_DEF,
-                "EL1_DEN" : dens,
-                "EL1_ROU" : self.EL1_ROU,
-                "EL1_FLAG" : self.EL1_FLAG,
-                "EL1_HGAP" : self.EL1_HGAP,
-                "EL1_VGAP" : self.EL1_VGAP,
-                "EL1_HGAPCENTER" : self.EL1_HGAPCENTER,
-                "EL1_VGAPCENTER" : self.EL1_VGAPCENTER,
-                "EL1_HMAG" : self.EL1_HMAG,
-                "EL1_VMAG" : self.EL1_VMAG,
-                "EL1_HROT" : self.EL1_HROT,
-                "EL1_VROT" : self.EL1_VROT,
-                "FILE_INPUT_NAME": "'"+self.FILE_INPUT_NAME+"'",
-                "INTERPOLATION_FLAG" : self.INTERPOLATION_FLAG,
-                "INTERPOLATION_FACTOR_H" : self.INTERPOLATION_FACTOR_H,
-                "INTERPOLATION_FACTOR_V" : self.INTERPOLATION_FACTOR_V,
-                "EL1_SLIT_CROP" : self.EL1_SLIT_CROP,
-            }
-
-
-
-        if self.input_beam is not None:
-            try:
-                script_previous = self.input_beam.get_content("xoppy_script")
-            except:
-                script_previous = '#\n# >> MISSING SCRIPT TO CREATE (energy, horizontal, vertical, flux3D) <<\n#\n'
-
-        script_element = self.script_template().format_map(dict_parameters)
-
-        script = script_previous + script_element
-        self.xoppy_script.set_code(script)
-
         return transmittance, absorbance, E, H, V, script
 
 
@@ -533,6 +750,9 @@ class OWpower3Dcomponent(XoppyWidget, WidgetDecorator):
 #
 
 import numpy
+try: import xraylib
+except: print("xraylib not available")
+from dabax.dabax_xraylib import DabaxXraylib
 from xoppylib.power.power3d import calculate_component_absorbance_and_transmittance
 from xoppylib.power.power3d import apply_transmittance_to_incident_beam
 
@@ -548,31 +768,38 @@ transmittance, absorbance, E, H, V, txt = calculate_component_absorbance_and_tra
                 defection={EL1_DEF},
                 dens={EL1_DEN},
                 roughness={EL1_ROU},
-                flags={EL1_FLAG},
-                hgap={EL1_HGAP},
-                vgap={EL1_VGAP},
-                hgapcenter={EL1_HGAPCENTER},
-                vgapcenter={EL1_VGAPCENTER},
-                hmag={EL1_HMAG},
-                vmag={EL1_VMAG},
-                hrot={EL1_HROT},
-                vrot={EL1_VROT},
+                flags={EL1_FLAG}, # 0=Filter 1=Mirror 2=Aperture 3=magnifier, 4=Screen rotation  5=Thin object  6=Multilayer 7=External file
+                hgap={hgap},
+                vgap={vgap},
+                hgapcenter={hgapcenter},
+                vgapcenter={vgapcenter},
+                hmag={hmag},
+                vmag={vmag},
+                hrot={hrot},
+                vrot={vrot},
+                thin_object_file='{thin_object_file}',
+                thin_object_thickness_outside_file_area={thin_object_thickness_outside_file_area},
+                thin_object_back_profile_flag={thin_object_back_profile_flag},
+                thin_object_back_profile_file='{thin_object_back_profile_file}',
+                multilayer_file='{multilayer_file}',
+                external_reflectivity_file='{external_reflectivity_file}',
+                material_constants_library = {material_constants_library},
                 )
 
 # apply transmittance to incident beam 
 f_transmitted, e, h, v = apply_transmittance_to_incident_beam(transmittance, f0, e0, h0, v0,
-                                  flags = {EL1_FLAG},
-                                  hgap = {EL1_HGAP},
-                                  vgap = {EL1_VGAP},
-                                  hgapcenter = {EL1_HGAPCENTER},
-                                  vgapcenter = {EL1_VGAPCENTER},
-                                  hmag = {EL1_HMAG},
-                                  vmag = {EL1_VMAG},
-                                  interpolation_flag     = {INTERPOLATION_FLAG},
-                                  interpolation_factor_h = {INTERPOLATION_FACTOR_H},
-                                  interpolation_factor_v = {INTERPOLATION_FACTOR_V},
-                                  slit_crop = {EL1_SLIT_CROP},
-                                )
+                flags = {EL1_FLAG},
+                hgap = {hgap},
+                vgap = {vgap},
+                hgapcenter = {hgapcenter},
+                vgapcenter = {vgapcenter},
+                hmag = {hmag},
+                vmag = {vmag},
+                interpolation_flag     = {INTERPOLATION_FLAG},
+                interpolation_factor_h = {INTERPOLATION_FACTOR_H},
+                interpolation_factor_v = {INTERPOLATION_FACTOR_V},
+                slit_crop = {EL1_SLIT_CROP},
+                )
 
 f_absorbed = f0 * absorbance / (H[0] / h0[0]) / (V[0] / v0[0])
 
@@ -582,34 +809,35 @@ energy, horizontal, vertical, flux3D = e, h, v, f_transmitted
 #                       
 # example plots
 #
-from srxraylib.plot.gol import plot_image
-import scipy.constants as codata
-from xoppylib.power.power3d import integral_2d
-
-# transmitted/reflected beam
-
-spectral_power_transmitted = f_transmitted * codata.e * 1e3     
-plot_image(spectral_power_transmitted[0,:,:],h,v,title="Transmitted Spectral Power Density [W/eV/mm2] at E=%g eV" % ({emin}),xtitle="H [mm]",ytitle="V [mm]",aspect='auto')
-
-power_density_transmitted = numpy.trapz(spectral_power_transmitted, e, axis=0)
-power_density_integral = integral_2d(power_density_transmitted, h, v)
-plot_image(power_density_transmitted, h, v,
-                 xtitle='H [mm] (normal to beam)',
-                 ytitle='V [mm] (normal to beam)',
-                 title='Power Density [W/mm^2]. Integral: %6.3f W'%power_density_integral,aspect='auto')
-
-# local absorption 
-
-spectral_power_density_absorbed = f_absorbed * codata.e * 1e3
-
-plot_image(spectral_power_density_absorbed[0,:,:],H,V,title="Absorbed Spectral Power Density [W/eV/mm2] at E=%g eV" % ({emin}),xtitle="H [mm]",ytitle="V [mm]",aspect='auto')
-
-power_density_absorbed = numpy.trapz(spectral_power_density_absorbed, E, axis=0)
-power_density_integral = integral_2d(power_density_absorbed, H, V)
-plot_image(power_density_absorbed, H, V,
-                 xtitle='H [mm] (o.e. coordinates)',
-                 ytitle='V [mm] (o.e. coordinates)',
-                 title='Absorbed Power Density [W/mm^2]. Integral: %6.3f W'%power_density_integral,aspect='auto')
+if True:
+    from srxraylib.plot.gol import plot_image
+    import scipy.constants as codata
+    from xoppylib.power.power3d import integral_2d
+    
+    # transmitted/reflected beam
+    
+    spectral_power_transmitted = f_transmitted * codata.e * 1e3     
+    plot_image(spectral_power_transmitted[0,:,:],h,v,title="Transmitted Spectral Power Density [W/eV/mm2] at E=%g eV" % ({emin}),xtitle="H [mm]",ytitle="V [mm]",aspect='auto')
+    
+    power_density_transmitted = numpy.trapz(spectral_power_transmitted, e, axis=0)
+    power_density_integral = integral_2d(power_density_transmitted, h, v)
+    plot_image(power_density_transmitted, h, v,
+                     xtitle='H [mm] (normal to beam)',
+                     ytitle='V [mm] (normal to beam)',
+                     title='Power Density [W/mm^2]. Integral: %6.3f W'%power_density_integral,aspect='auto')
+    
+    # local absorption 
+    
+    spectral_power_density_absorbed = f_absorbed * codata.e * 1e3
+    
+    plot_image(spectral_power_density_absorbed[0,:,:],H,V,title="Absorbed Spectral Power Density [W/eV/mm2] at E=%g eV" % ({emin}),xtitle="H [mm]",ytitle="V [mm]",aspect='auto')
+    
+    power_density_absorbed = numpy.trapz(spectral_power_density_absorbed, E, axis=0)
+    power_density_integral = integral_2d(power_density_absorbed, H, V)
+    plot_image(power_density_absorbed, H, V,
+                     xtitle='H [mm] (o.e. coordinates)',
+                     ytitle='V [mm] (o.e. coordinates)',
+                     title='Absorbed Power Density [W/mm^2]. Integral: %6.3f W'%power_density_integral,aspect='auto')
                                                
 #
 # end script
@@ -654,7 +882,7 @@ plot_image(power_density_absorbed, H, V,
 
     def getTitles(self):
         mylist = []
-        if self.EL1_FLAG == 1:
+        if self.EL1_FLAG in (1, 6):
             txt1 = "Reflectance "
             txt2 = "Reflected "
         else:
@@ -987,8 +1215,6 @@ if __name__ == "__main__":
     w.EL1_VGAPCENTER = 0.0
     w.PLOT_SETS = 3
 
-
-
     w.show()
     app.exec()
     w.saveSettings()
