@@ -5,17 +5,21 @@ from orangewidget.settings import Setting
 from oasys.widgets import gui as oasysgui, congruence
 
 from oasys.widgets.exchange import DataExchangeObject
-from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
+from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget_dabax import XoppyWidgetDabax
 
 
 import scipy.constants as codata
 
-import xraylib
+try: import xraylib
+except: print("xraylib not available")
+
 from dabax.dabax_xraylib import DabaxXraylib
+from dabax.dabax_files import dabax_f1f2_files
 
 from xoppylib.crystals.tools import run_diff_pat, bragg_calc2
 
-class OWxcrystal(XoppyWidget):
+
+class OWxcrystal(XoppyWidgetDabax):
     name = "CRYSTAL"
     id = "orange.widgets.dataxcrystal"
     description = "Crystal Reflectivity (perfect, bent, mosaic)"
@@ -49,16 +53,24 @@ class OWxcrystal(XoppyWidget):
 
     # new crystals  #todo: add to menus?
     material_constants_library_flag = Setting(2) # 0=xraylib, 1=dabax, 2=xraylib completed by dabax
-    dx = None # DABAX object
 
 
     def __init__(self):
         super().__init__(show_script_tab=True)
 
+    def dabax_show_f1f2(self):
+        return True
+
     def build_gui(self):
 
-        box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
-        
+        # box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
+        ###########
+        tabs_setting = oasysgui.tabWidget(self.controlArea)
+        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)
+        box = oasysgui.createTabPage(tabs_setting, self.name + " Input Parameters")
+        self.tab_dabax = oasysgui.createTabPage(tabs_setting, "Materials Library")
+        ###########
+
         idx = -1 
         
         #widget index 3 
@@ -284,10 +296,10 @@ class OWxcrystal(XoppyWidget):
         congruence.checkLessThan(self.SCANFROM, self.SCANTO, "Min Scan value", "Max Scan value")
         self.SCANPOINTS = congruence.checkStrictlyPositiveNumber(self.SCANPOINTS, "Scan points")
 
-        if self.SCAN < 4:
-            self.ENERGY = congruence.checkStrictlyPositiveNumber(self.ENERGY , "Fix value")
+        if self.SCAN == 3:
+            self.ENERGY = congruence.checkNumber(self.ENERGY, "Fix value")
         else:
-            self.ENERGY = congruence.checkNumber(self.ENERGY , "Fix value")
+            self.ENERGY = congruence.checkStrictlyPositiveNumber(self.ENERGY , "Fix value")
 
         if self.MOSAIC == 0: #perfect
             self.ASYMMETRY_ANGLE = congruence.checkNumber(self.ASYMMETRY_ANGLE, "Asymmetry angle")
@@ -309,42 +321,47 @@ class OWxcrystal(XoppyWidget):
                 congruence.checkFile(self.FILECOMPLIANCE)
 
     def get_crystal_list(self):
-        crystal_list_xrl = list(xraylib.Crystal_GetCrystalsList())
+        return list(DabaxXraylib().Crystal_GetCrystalsList())
 
-        if self.material_constants_library_flag == 0:
-            return crystal_list_xrl
-
-        if self.dx is None:
-            self.dx = DabaxXraylib()
-        crystal_list_dabax = self.dx.Crystal_GetCrystalsList()
-
-        if self.material_constants_library_flag == 1:
-            return crystal_list_dabax
-
-        crystal_list_combined = crystal_list_xrl
-        for crystal in crystal_list_dabax:
-            if crystal not in crystal_list_combined:
-                crystal_list_combined.append(crystal)
-
-        if self.material_constants_library_flag == 2:
-            return crystal_list_combined
+        # crystal_list_xrl = list(xraylib.Crystal_GetCrystalsList())
+        #
+        # if self.material_constants_library_flag == 0:
+        #     return crystal_list_xrl
+        #
+        # if self.dx is None:
+        #     self.dx = DabaxXraylib()
+        # crystal_list_dabax = self.dx.Crystal_GetCrystalsList()
+        #
+        # if self.material_constants_library_flag == 1:
+        #     return crystal_list_dabax
+        #
+        # crystal_list_combined = crystal_list_xrl
+        # for crystal in crystal_list_dabax:
+        #     if crystal not in crystal_list_combined:
+        #         crystal_list_combined.append(crystal)
+        #
+        # if self.material_constants_library_flag == 2:
+        #     return crystal_list_combined
 
     def do_xoppy_calculation(self):
         # return self.xoppy_calc_xcrystal()
 
         descriptor = self.get_crystal_list()[self.CRYSTAL_MATERIAL]
+        print("Using crystal descriptor: ",descriptor)
 
-        if self.material_constants_library_flag == 0:
+        if self.MATERIAL_CONSTANT_LIBRARY_FLAG == 0:
             material_constants_library = xraylib
-        elif self.material_constants_library_flag == 1:
-            material_constants_library = self.dx
-        elif self.material_constants_library_flag == 2:
-            if descriptor in xraylib.Crystal_GetCrystalsList():
-                material_constants_library = xraylib
-            elif descriptor in self.dx.Crystal_GetCrystalsList():
-                material_constants_library = self.dx
-            else:
-                raise Exception("Descriptor not found in material constants database")
+        elif self.MATERIAL_CONSTANT_LIBRARY_FLAG == 1:
+            material_constants_library = DabaxXraylib(file_f1f2=dabax_f1f2_files()[self.DABAX_F1F2_FILE_INDEX])
+            print(material_constants_library.info())
+
+        # elif self.material_constants_library_flag == 2:
+        #     if descriptor in xraylib.Crystal_GetCrystalsList():
+        #         material_constants_library = xraylib
+        #     elif descriptor in self.dx.Crystal_GetCrystalsList():
+        #         material_constants_library = self.dx
+        #     else:
+        #         raise Exception("Descriptor not found in material constants database")
 
         #
         # run bragg_calc (preprocessor) and create file xcrystal.bra
@@ -368,6 +385,9 @@ class OWxcrystal(XoppyWidget):
 
         print("Using crystal descriptor: ", descriptor)
 
+        #
+        # run
+        #
         bragg_dictionary = bragg_calc2(
             descriptor=descriptor,
             hh=self.MILLER_INDEX_H,
@@ -383,43 +403,12 @@ class OWxcrystal(XoppyWidget):
             verbose=False,
             material_constants_library=material_constants_library,
         )
-
-        #
-        # run external (fortran) diff_pat
-        #
-        run_diff_pat(
-            bragg_dictionary,
-            preprocessor_file=preprocessor_file,
-            descriptor=descriptor,
-            MOSAIC=self.MOSAIC,
-            GEOMETRY=self.GEOMETRY,
-            SCAN=self.SCAN,
-            UNIT=self.UNIT,
-            SCANFROM=self.SCANFROM,
-            SCANTO=self.SCANTO,
-            SCANPOINTS=self.SCANPOINTS,
-            ENERGY=self.ENERGY,
-            ASYMMETRY_ANGLE=self.ASYMMETRY_ANGLE,
-            THICKNESS=self.THICKNESS,
-            MOSAIC_FWHM=self.MOSAIC_FWHM,
-            RSAG=self.RSAG,
-            RMER=self.RMER,
-            ANISOTROPY=self.ANISOTROPY,
-            POISSON=self.POISSON,
-            CUT=self.CUT,
-            FILECOMPLIANCE=self.FILECOMPLIANCE,
-        )
-
-        # show calculated parameters in standard output
-        txt_info = open("diff_pat.par").read()
-        for line in txt_info:
-            print(line, end="")
-
         #
         # write python script
         #
         if isinstance(material_constants_library, DabaxXraylib):
-            material_constants_library_txt = "DabaxXraylib()"
+            material_constants_library_txt = 'DabaxXraylib(file_f1f2="%s")' % (
+            dabax_f1f2_files()[self.DABAX_F1F2_FILE_INDEX])
         else:
             material_constants_library_txt = "xraylib"
 
@@ -455,6 +444,37 @@ class OWxcrystal(XoppyWidget):
         script = self.script_template().format_map(dict_parameters)
         self.xoppy_script.set_code(script)
 
+        #
+        # run external (fortran) diff_pat
+        #
+        run_diff_pat(
+            bragg_dictionary,
+            preprocessor_file=preprocessor_file,
+            descriptor=descriptor,
+            MOSAIC=self.MOSAIC,
+            GEOMETRY=self.GEOMETRY,
+            SCAN=self.SCAN,
+            UNIT=self.UNIT,
+            SCANFROM=self.SCANFROM,
+            SCANTO=self.SCANTO,
+            SCANPOINTS=self.SCANPOINTS,
+            ENERGY=self.ENERGY,
+            ASYMMETRY_ANGLE=self.ASYMMETRY_ANGLE,
+            THICKNESS=self.THICKNESS,
+            MOSAIC_FWHM=self.MOSAIC_FWHM,
+            RSAG=self.RSAG,
+            RMER=self.RMER,
+            ANISOTROPY=self.ANISOTROPY,
+            POISSON=self.POISSON,
+            CUT=self.CUT,
+            FILECOMPLIANCE=self.FILECOMPLIANCE,
+        )
+
+        # show calculated parameters in standard output
+        txt_info = open("diff_pat.par").read()
+        for line in txt_info:
+            print(line, end="")
+
         return bragg_dictionary, "diff_pat.dat", script
 
     def get_units_to_degrees(self):
@@ -475,7 +495,8 @@ class OWxcrystal(XoppyWidget):
 
 import numpy
 from xoppylib.crystals.tools import bragg_calc2, run_diff_pat
-import xraylib
+try: import xraylib
+except: print("xraylib not available")
 from dabax.dabax_xraylib import DabaxXraylib
 
 #
@@ -526,9 +547,10 @@ run_diff_pat(
 #                       
 # example plot
 #
-from srxraylib.plot.gol import plot
-data = numpy.loadtxt("diff_pat.dat", skiprows=5)
-plot(data[:,0], data[:,-1], data[:,0], data[:,-2], ytitle='Crystal reflectivity', legend=['s-polarization','p-polarization'])
+if True:
+    from srxraylib.plot.gol import plot
+    data = numpy.loadtxt("diff_pat.dat", skiprows=5)
+    plot(data[:,0], data[:,-1], data[:,0], data[:,-2], ytitle='Crystal reflectivity', legend=['s-polarization','p-polarization'])
 
 #
 # end script
@@ -556,6 +578,15 @@ plot(data[:,0], data[:,-1], data[:,0], data[:,-2], ytitle='Crystal reflectivity'
                 calculated_data.add_content("bragg_angle", numpy.degrees(numpy.arcsin(wavelength / (2 * dspacing))))
                 calculated_data.add_content("asymmetry_angle", self.ASYMMETRY_ANGLE)
 
+            if self.SCAN == 3:
+                if self.ENERGY > 0:
+                    calculated_data.add_content("bragg_angle", self.ENERGY)
+                else:
+                    wavelength = codata.h * codata.c / codata.e / numpy.abs(self.ENERGY) * 1e2  # cm
+                    dspacing = float(bragg_dictionary["dspacing"])
+                    calculated_data.add_content("bragg_angle", numpy.degrees(numpy.arcsin(wavelength / (2 * dspacing))))
+                calculated_data.add_content("asymmetry_angle", self.ASYMMETRY_ANGLE)
+
             calculated_data.add_content("units_to_degrees", self.get_units_to_degrees())
         except Exception as e:
             raise Exception("Error loading diff_pat.dat :" + str(e))
@@ -644,258 +675,6 @@ plot(data[:,0], data[:,-1], data[:,0], data[:,-2], ytitle='Crystal reflectivity'
                                                           text="Peak=%5.2f"%(y[index_ymax]), color="pink",
                                                           selectable=False, draggable=False, symbol=None, constraint=None)
 
-#     def xoppy_calc_xcrystal(self):
-#
-#         descriptor = self.get_crystal_list()[self.CRYSTAL_MATERIAL]
-#         print(">>>>>> descriptor 0: ", descriptor)
-#
-#         if self.material_constants_library_flag == 0:
-#             material_constants_library = xraylib
-#         elif self.material_constants_library_flag == 1:
-#             material_constants_library = self.dx
-#         elif self.material_constants_library_flag == 2:
-#             if descriptor in xraylib.Crystal_GetCrystalsList():
-#                 material_constants_library = xraylib
-#             elif descriptor in self.dx.Crystal_GetCrystalsList():
-#                 material_constants_library = self.dx
-#             else:
-#                 raise Exception("Descriptor not found in material constants database")
-#
-#         #
-#         # run bragg_calc (preprocessor) and create file xcrystal.bra
-#         #
-#
-#         # for file in ["xcrystal.bra"]:
-#         #     try:
-#         #         os.remove(os.path.join(locations.home_bin_run(), file))
-#         #     except:
-#         #         pass
-#
-#         if self.SCAN == 3:  # energy scan
-#             emin = self.SCANFROM - 1
-#             emax = self.SCANTO + 1
-#         else:
-#             emin = self.ENERGY - 100.0
-#             emax = self.ENERGY + 100.0
-#
-#         estep = (emax - emin) / self.SCANPOINTS
-#         preprocessor_file = "xcrystal.bra"
-#
-#         print("Using crystal descriptor: ", descriptor)
-#
-#         bragg_dictionary = bragg_calc2(
-#             descriptor=descriptor,
-#             hh = self.MILLER_INDEX_H,
-#             kk = self.MILLER_INDEX_K,
-#             ll = self.MILLER_INDEX_L,
-#             temper = float(self.TEMPER),
-#             emin=emin,
-#             emax=emax,
-#             estep=estep,
-#             ANISO_SEL=0,
-#             fileout=preprocessor_file,
-#             do_not_prototype=0,  # 0=use site groups (recommended), 1=use all individual sites
-#             verbose=False,
-#             material_constants_library=material_constants_library,
-#         )
-#
-#         print(">>>>>> descriptor 1: ", descriptor)
-#         #
-#         # run external (fortran) diff_pat
-#         #
-#         run_diff_pat(
-#             bragg_dictionary,
-#             preprocessor_file=preprocessor_file,
-#             descriptor=descriptor,
-#             MOSAIC = self.MOSAIC,
-#             GEOMETRY = self.GEOMETRY,
-#             SCAN = self.SCAN,
-#             UNIT = self.UNIT,
-#             SCANFROM = self.SCANFROM,
-#             SCANTO = self.SCANTO,
-#             SCANPOINTS = self.SCANPOINTS,
-#             ENERGY = self.ENERGY,
-#             ASYMMETRY_ANGLE = self.ASYMMETRY_ANGLE,
-#             THICKNESS = self.THICKNESS,
-#             MOSAIC_FWHM = self.MOSAIC_FWHM,
-#             RSAG = self.RSAG,
-#             RMER = self.RMER,
-#             ANISOTROPY = self.ANISOTROPY,
-#             POISSON = self.POISSON,
-#             CUT = self.CUT,
-#             FILECOMPLIANCE = self.FILECOMPLIANCE,
-#             )
-#
-#         print(">>>>>> descriptor 2: ", descriptor)
-#         #show calculated parameters in standard output
-#         txt_info = open("diff_pat.par").read()
-#         for line in txt_info:
-#             print(line,end="")
-#
-#         #
-#         # write python script
-#         #
-#         if isinstance(material_constants_library, DabaxXraylib):
-#             material_constants_library_txt = "DabaxXraylib()"
-#         else:
-#             material_constants_library_txt = "xraylib"
-#
-#
-#         dict_parameters = {
-#             'CRYSTAL_DESCRIPTOR': descriptor,
-#             'MILLER_INDEX_H': self.MILLER_INDEX_H,
-#             'MILLER_INDEX_K': self.MILLER_INDEX_K,
-#             'MILLER_INDEX_L': self.MILLER_INDEX_L,
-#             'TEMPER': self.TEMPER,
-#             'MOSAIC': self.MOSAIC,
-#             'GEOMETRY': self.GEOMETRY,
-#             'SCAN': self.SCAN,
-#             'UNIT': self.UNIT,
-#             'SCANFROM': self.SCANFROM,
-#             'SCANTO': self.SCANTO,
-#             'SCANPOINTS': self.SCANPOINTS,
-#             'ENERGY': self.ENERGY,
-#             'ASYMMETRY_ANGLE': self.ASYMMETRY_ANGLE,
-#             'THICKNESS': self.THICKNESS,
-#             'MOSAIC_FWHM': self.MOSAIC_FWHM,
-#             'RSAG': self.RSAG,
-#             'RMER': self.RMER,
-#             'ANISOTROPY': self.ANISOTROPY,
-#             'POISSON': self.POISSON,
-#             'CUT': self.CUT,
-#             'FILECOMPLIANCE': self.FILECOMPLIANCE,
-#             'material_constants_library_txt': material_constants_library_txt,
-#             'emin': emin,
-#             'emax': emax,
-#             'estep': estep,
-#             'preprocessor_file': preprocessor_file}
-#
-#         self.xoppy_script.set_code(self.script_template().format_map(dict_parameters))
-#
-#
-#         #
-#         # prepare outputs
-#         #
-#
-#         calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
-#
-#         try:
-#             calculated_data.add_content("xoppy_data", numpy.loadtxt("diff_pat.dat", skiprows=5))
-#             calculated_data.add_content("plot_x_col",0)
-#             calculated_data.add_content("plot_y_col",-1)
-#             calculated_data.add_content("scan_type", self.SCAN)
-#
-#             if self.SCAN in (1, 2):
-#                 wavelength = codata.h * codata.c / codata.e / self.ENERGY * 1e2 # cm
-#                 dspacing = float(bragg_dictionary["dspacing"])
-#
-#                 calculated_data.add_content("bragg_angle", numpy.degrees(numpy.arcsin(wavelength/(2*dspacing))))
-#                 calculated_data.add_content("asymmetry_angle", self.ASYMMETRY_ANGLE)
-#
-#             calculated_data.add_content("units_to_degrees", self.get_units_to_degrees())
-#         except Exception as e:
-#             raise Exception("Error loading diff_pat.dat :" + str(e))
-#
-#         try:
-#             calculated_data.add_content("labels",
-#                                         ["Th-ThB{in} [" + self.unit_combo.itemText(self.UNIT) + "]",
-#                                          "Th-ThB{out} [" + self.unit_combo.itemText(self.UNIT) + "]",
-#                                          "phase_p[rad]",
-#                                          "phase_s[rad]","Circ Polariz",
-#                                          "p-polarized reflectivity",
-#                                          "s-polarized reflectivity"])
-#
-#         except:
-#             pass
-#
-#         try:
-#             with open("diff_pat.par") as f:
-#                 info = f.readlines()
-#             calculated_data.add_content("info",info)
-#         except:
-#             pass
-#
-#         return calculated_data
-#
-#
-#     def get_units_to_degrees(self):
-#         if self.UNIT == 0: # RADIANS
-#             return 57.2957795
-#         elif self.UNIT == 1: #MICRORADIANS
-#             return 57.2957795e-6
-#         elif self.UNIT == 2: # DEGREES
-#             return 1.0
-#         elif self.UNIT == 3: # ARCSEC
-#             return 0.000277777805
-#
-#     def script_template(self):
-#         return """
-# #
-# # script to calculate crystal diffraction profiles (created by XOPPY:crystal)
-# #
-#
-# import numpy
-# from xoppylib.crystals.tools import bragg_calc2, run_diff_pat
-# import xraylib
-# from dabax.dabax_xraylib import DabaxXraylib
-#
-# #
-# # run bragg_calc (preprocessor) and create file xcrystal.bra
-# #
-# bragg_dictionary = bragg_calc2(
-#     descriptor = "{CRYSTAL_DESCRIPTOR}",
-#     hh         = {MILLER_INDEX_H},
-#     kk         = {MILLER_INDEX_K},
-#     ll         = {MILLER_INDEX_L},
-#     temper     = {TEMPER},
-#     emin       = {emin},
-#     emax       = {emax},
-#     estep      = {estep},
-#     ANISO_SEL  = 0,
-#     fileout    = "{preprocessor_file}",
-#     do_not_prototype = 0,  # 0=use site groups (recommended), 1=use all individual sites
-#     verbose = False,
-#     material_constants_library = {material_constants_library_txt},
-#     )
-#
-# #
-# # run external (fortran) diff_pat (note that some parameters may not be used)
-# #
-# run_diff_pat(
-#     bragg_dictionary,
-#     preprocessor_file  = "{preprocessor_file}",
-#     descriptor         = "{CRYSTAL_DESCRIPTOR}",
-#     MOSAIC             = {MOSAIC},
-#     GEOMETRY           = {GEOMETRY},
-#     SCAN               = {SCAN},
-#     UNIT               = {UNIT},
-#     SCANFROM           = {SCANFROM},
-#     SCANTO             = {SCANTO},
-#     SCANPOINTS         = {SCANPOINTS},
-#     ENERGY             = {ENERGY},
-#     ASYMMETRY_ANGLE    = {ASYMMETRY_ANGLE},
-#     THICKNESS          = {THICKNESS},
-#     MOSAIC_FWHM        = {MOSAIC_FWHM},
-#     RSAG               = {RSAG},
-#     RMER               = {RMER},
-#     ANISOTROPY         = {ANISOTROPY},
-#     POISSON            = {POISSON},
-#     CUT                = "{CUT}",
-#     FILECOMPLIANCE     = "{FILECOMPLIANCE}",
-#     )
-#
-# #
-# # example plot
-# #
-# from srxraylib.plot.gol import plot
-# data = numpy.loadtxt("diff_pat.dat", skiprows=5)
-# plot(data[:,0], data[:,-1], data[:,0], data[:,-2], ytitle='Crystal reflectivity', legend=['s-polarization','p-polarization'])
-#
-# #
-# # end script
-# #
-# """
-
 #
 #
 #
