@@ -7,17 +7,20 @@ from oasys.widgets import gui as oasysgui, congruence
 from oasys.widgets.exchange import DataExchangeObject
 
 
-from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
+from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget_dabax import XoppyWidgetDabax
+
 
-from dabax.dabax_files import dabax_f0_files
 
 from xoppylib.scattering_functions.xoppy_calc_f0 import xoppy_calc_f0
 
-import xraylib
+try: import xraylib
+except: print("xraylib not available")
+
+from dabax.dabax_files import dabax_f0_files
 from dabax.dabax_xraylib import DabaxXraylib
 
 
-class OWxf0(XoppyWidget):
+class OWxf0(XoppyWidgetDabax):
     name = "F0"
     id = "orange.widgets.dataxf0"
     description = "Elastic Scattering Function"
@@ -34,17 +37,24 @@ class OWxf0(XoppyWidget):
     DUMP_TO_FILE = Setting(0)  # No
     FILE_NAME    = Setting("f0.dat")
     CHARGE       = Setting(0.0)
-    MATERIAL_CONSTANT_LIBRARY_FLAG = Setting(0)
+    # MATERIAL_CONSTANT_LIBRARY_FLAG = Setting(0)
 
     NIST_NAME    = Setting(177)
-    DABAX_F0_FILE_INDEX = Setting(0)
+    # DABAX_F0_FILE_INDEX = Setting(0)
 
     def __init__(self):
         super().__init__(show_script_tab=True)
 
-    def build_gui(self):
+    def dabax_show_f0(self):
+        return True
 
-        box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
+    def build_gui(self):
+        ###########
+        tabs_setting = oasysgui.tabWidget(self.controlArea)
+        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)
+        box = oasysgui.createTabPage(tabs_setting, self.name + " Input Parameters")
+        self.tab_dabax = oasysgui.createTabPage(tabs_setting, "Materials Library")
+        ###########
         
         idx = -1 
 
@@ -68,7 +78,7 @@ class OWxf0(XoppyWidget):
         #widget index 3
         idx += 1
         box1 = gui.widgetBox(box)
-        self.nist_list = xraylib.GetCompoundDataNISTList()
+        self.nist_list = DabaxXraylib().GetCompoundDataNISTList()
         gui.comboBox(box1, self, "NIST_NAME",
                      label=self.unitLabels()[idx], addSpace=False,
                     items=self.nist_list,
@@ -116,27 +126,6 @@ class OWxf0(XoppyWidget):
                      label=self.unitLabels()[idx], addSpace=True)
         self.show_at(self.unitFlags()[idx], box1)
 
-
-        box = oasysgui.widgetBox(self.controlArea, self.name + " Material Library", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
-
-        # widget index 10
-        idx += 1
-        box1 = gui.widgetBox(box)
-        gui.comboBox(box1, self, "MATERIAL_CONSTANT_LIBRARY_FLAG",
-                     label=self.unitLabels()[idx], addSpace=True,
-                     items=["xraylib [default]", "dabax"],
-                     orientation="horizontal")
-        self.show_at(self.unitFlags()[idx], box1)
-
-        # widget index 11
-        idx += 1
-        box1 = gui.widgetBox(box)
-        gui.comboBox(box1, self, "DABAX_F0_FILE_INDEX",
-                     label=self.unitLabels()[idx], addSpace=True,
-                     items=dabax_f0_files(),
-                     orientation="horizontal")
-        self.show_at(self.unitFlags()[idx], box1)
-
         gui.rubber(self.controlArea)
 
     def unitLabels(self):
@@ -159,7 +148,6 @@ class OWxf0(XoppyWidget):
 
     def do_xoppy_calculation(self):
 
-
         if self.MAT_FLAG == 2:
             descriptor = self.nist_list[self.NIST_NAME]
         else:
@@ -173,21 +161,9 @@ class OWxf0(XoppyWidget):
             material_constants_library_str = 'DabaxXraylib(file_f0="%s")' % (dabax_f0_files()[self.DABAX_F0_FILE_INDEX])
             print(material_constants_library.info())
 
-        out_dict = xoppy_calc_f0(
-                descriptor                  = descriptor,
-                MAT_FLAG                    = self.MAT_FLAG,
-                GRIDSTART                   = self.GRIDSTART,
-                GRIDEND                     = self.GRIDEND,
-                GRIDN                       = self.GRIDN,
-                DUMP_TO_FILE                = self.DUMP_TO_FILE,
-                FILE_NAME                   = self.FILE_NAME,
-                CHARGE                      = 0.0,
-                material_constants_library  = material_constants_library,
-                )
-
-        if "info" in out_dict.keys():
-            print(out_dict["info"])
-
+        #
+        # script
+        #
         dict_parameters = {
                 "descriptor"                      : descriptor,
                 "MAT_FLAG"                        : self.MAT_FLAG,
@@ -204,6 +180,24 @@ class OWxf0(XoppyWidget):
 
         self.xoppy_script.set_code(script)
 
+        #
+        # run
+        #
+        out_dict = xoppy_calc_f0(
+                descriptor                  = descriptor,
+                MAT_FLAG                    = self.MAT_FLAG,
+                GRIDSTART                   = self.GRIDSTART,
+                GRIDEND                     = self.GRIDEND,
+                GRIDN                       = self.GRIDN,
+                DUMP_TO_FILE                = self.DUMP_TO_FILE,
+                FILE_NAME                   = self.FILE_NAME,
+                CHARGE                      = 0.0,
+                material_constants_library  = material_constants_library,
+                )
+
+        if "info" in out_dict.keys():
+            print(out_dict["info"])
+
         return out_dict
 
     def script_template(self):
@@ -212,7 +206,8 @@ class OWxf0(XoppyWidget):
 # script to make the calculations (created by XOPPY:xf0)
 #
 from xoppylib.scattering_functions.xoppy_calc_f0 import xoppy_calc_f0
-import xraylib
+try: import xraylib
+except: print("xraylib not available")
 from dabax.dabax_xraylib import DabaxXraylib
 
 out_dict =  xoppy_calc_f0(
@@ -230,10 +225,11 @@ out_dict =  xoppy_calc_f0(
 #
 # example plot
 #
-from srxraylib.plot.gol import plot
-plot(out_dict["data"][0,:],out_dict["data"][-1,:],
-    xtitle=out_dict["labels"][0],ytitle=out_dict["labels"][1],title="f0",
-    xlog=False,ylog=False,show=True)
+if True:
+    from srxraylib.plot.gol import plot
+    plot(out_dict["data"][0,:],out_dict["data"][-1,:],
+        xtitle=out_dict["labels"][0],ytitle=out_dict["labels"][1],title="f0",
+        xlog=False,ylog=False,show=True)
 #
 # end script
 #
