@@ -6,7 +6,7 @@ from orangewidget.settings import Setting
 from oasys.widgets import gui as oasysgui, congruence
 from oasys.widgets.exchange import DataExchangeObject
 
-from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
+from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget_dabax import XoppyWidgetDabax
 
 from xoppylib.scattering_functions.xoppy_calc_f1f2 import xoppy_calc_f1f2
 from dabax.dabax_files import dabax_f1f2_files
@@ -15,7 +15,7 @@ import xraylib
 from dabax.dabax_xraylib import DabaxXraylib
 
 
-class OWxf1f2(XoppyWidget):
+class OWxf1f2(XoppyWidgetDabax):
     name = "F1F2"
     id = "orange.widgets.dataxf1f2"
     description = "X-ray Matter Scattering Functions and Reflectivity"
@@ -42,8 +42,8 @@ class OWxf1f2(XoppyWidget):
     DUMP_TO_FILE = Setting(0)  # No
     FILE_NAME    = Setting("f1f2.dat")
 
-    MATERIAL_CONSTANT_LIBRARY_FLAG = Setting(0)
-    DABAX_F1F2_FILE_INDEX = Setting(0)
+    # MATERIAL_CONSTANT_LIBRARY_FLAG = Setting(0)
+    # DABAX_F1F2_FILE_INDEX = Setting(0)
 
     xtitle = None
     ytitle = None
@@ -51,10 +51,20 @@ class OWxf1f2(XoppyWidget):
     def __init__(self):
         super().__init__(show_script_tab=True)
 
+    def dabax_show_f1f2(self):
+        return True
+
     def build_gui(self):
 
-        box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
-        
+        # box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
+        ###########
+        tabs_setting = oasysgui.tabWidget(self.controlArea)
+        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)
+        box = oasysgui.createTabPage(tabs_setting, self.name + " Input Parameters")
+        self.tab_dabax = oasysgui.createTabPage(tabs_setting, "Materials Library")
+        ###########
+
+
         idx = -1
         
         #widget index 1 
@@ -204,26 +214,26 @@ class OWxf1f2(XoppyWidget):
         #
         #
 
-        box = oasysgui.widgetBox(self.controlArea, self.name + " Material Library", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
+        # box = oasysgui.widgetBox(self.controlArea, self.name + " Material Library", orientation="vertical", width=self.CONTROL_AREA_WIDTH-5)
+        #
+        #
+        # # widget index 17
+        # idx += 1
+        # box1 = gui.widgetBox(box)
+        # gui.comboBox(box1, self, "MATERIAL_CONSTANT_LIBRARY_FLAG",
+        #              label=self.unitLabels()[idx], addSpace=True,
+        #              items=["xraylib [default]", "dabax"],
+        #              orientation="horizontal")
+        # self.show_at(self.unitFlags()[idx], box1)
 
-
-        # widget index 17
-        idx += 1
-        box1 = gui.widgetBox(box)
-        gui.comboBox(box1, self, "MATERIAL_CONSTANT_LIBRARY_FLAG",
-                     label=self.unitLabels()[idx], addSpace=True,
-                     items=["xraylib [default]", "dabax"],
-                     orientation="horizontal")
-        self.show_at(self.unitFlags()[idx], box1)
-
-        # widget index 18
-        idx += 1
-        box1 = gui.widgetBox(box)
-        gui.comboBox(box1, self, "DABAX_F1F2_FILE_INDEX",
-                     label=self.unitLabels()[idx], addSpace=True,
-                     items=dabax_f1f2_files(),
-                     orientation="horizontal")
-        self.show_at(self.unitFlags()[idx], box1)
+        # # widget index 18
+        # idx += 1
+        # box1 = gui.widgetBox(box)
+        # gui.comboBox(box1, self, "DABAX_F1F2_FILE_INDEX",
+        #              label=self.unitLabels()[idx], addSpace=True,
+        #              items=dabax_f1f2_files(),
+        #              orientation="horizontal")
+        # self.show_at(self.unitFlags()[idx], box1)
 
         gui.rubber(self.controlArea)
 
@@ -245,9 +255,7 @@ class OWxf1f2(XoppyWidget):
                  'To [mrad]',  #   (self.CALCULATE  ==  0 or (
                  'Number of angular points',  #   (self.CALCULATE  ==  0 or (
                  'Dump to file',
-                 'File name',
-                 'Material Library',
-                 'dabax f1f2 file']
+                 'File name']
 
 
     def unitFlags(self):
@@ -266,9 +274,7 @@ class OWxf1f2(XoppyWidget):
                  '(self.CALCULATE  == 7 or  self.CALCULATE == 8 or self.CALCULATE  == 9)  &  self.THETAGRID  ==  1',
                  '(self.CALCULATE  == 7 or  self.CALCULATE == 8 or self.CALCULATE  == 9)  &  self.THETAGRID  ==  1',
                  'True',
-                 'self.DUMP_TO_FILE == 1',
-                 'True',
-                 'self.MATERIAL_CONSTANT_LIBRARY_FLAG == 1']
+                 'self.DUMP_TO_FILE == 1']
 
     def get_help_name(self):
         return 'f1f2'
@@ -333,28 +339,9 @@ class OWxf1f2(XoppyWidget):
         print("Using descriptor: %s" % descriptor)
         print("Using density: %6.3f" % density)
 
-        out_dict = xoppy_calc_f1f2(
-            descriptor                 = descriptor  ,
-            density                    = density     ,
-            MAT_FLAG                   = self.MAT_FLAG    ,
-            CALCULATE                  = self.CALCULATE   ,
-            GRID                       = self.GRID        ,
-            GRIDSTART                  = self.GRIDSTART   ,
-            GRIDEND                    = self.GRIDEND     ,
-            GRIDN                      = self.GRIDN       ,
-            THETAGRID                  = self.THETAGRID   ,
-            ROUGH                      = self.ROUGH       ,
-            THETA1                     = self.THETA1      ,
-            THETA2                     = self.THETA2      ,
-            THETAN                     = self.THETAN      ,
-            DUMP_TO_FILE               = self.DUMP_TO_FILE,
-            FILE_NAME                  = self.FILE_NAME   ,
-            material_constants_library = material_constants_library,
-        )
-
-        if "info" in out_dict.keys():
-            print(out_dict["info"])
-
+        #
+        # script
+        #
         dict_parameters = {
             "descriptor"                 : descriptor  ,
             "density"                    : density     ,
@@ -378,6 +365,31 @@ class OWxf1f2(XoppyWidget):
 
         self.xoppy_script.set_code(script)
 
+        #
+        # run
+        #
+        out_dict = xoppy_calc_f1f2(
+            descriptor                 = descriptor  ,
+            density                    = density     ,
+            MAT_FLAG                   = self.MAT_FLAG    ,
+            CALCULATE                  = self.CALCULATE   ,
+            GRID                       = self.GRID        ,
+            GRIDSTART                  = self.GRIDSTART   ,
+            GRIDEND                    = self.GRIDEND     ,
+            GRIDN                      = self.GRIDN       ,
+            THETAGRID                  = self.THETAGRID   ,
+            ROUGH                      = self.ROUGH       ,
+            THETA1                     = self.THETA1      ,
+            THETA2                     = self.THETA2      ,
+            THETAN                     = self.THETAN      ,
+            DUMP_TO_FILE               = self.DUMP_TO_FILE,
+            FILE_NAME                  = self.FILE_NAME   ,
+            material_constants_library = material_constants_library,
+        )
+
+        if "info" in out_dict.keys():
+            print(out_dict["info"])
+
         return out_dict
 
     def script_template(self):
@@ -386,7 +398,8 @@ class OWxf1f2(XoppyWidget):
 # script to make the calculations (created by XOPPY:xf1f2)
 #
 from xoppylib.scattering_functions.xoppy_calc_f1f2 import xoppy_calc_f1f2
-import xraylib
+try: import xraylib
+except: print("xraylib not available")
 from dabax.dabax_xraylib import DabaxXraylib
 
 out_dict =  xoppy_calc_f1f2(
@@ -411,15 +424,16 @@ out_dict =  xoppy_calc_f1f2(
 #
 # example plot
 #
-from srxraylib.plot.gol import plot, plot_image
-try:
-    plot(out_dict["data"][0,:],out_dict["data"][-1,:],
-        xtitle=out_dict["labels"][0],ytitle=out_dict["labels"][1],title="xf1f2",
-        xlog=True,ylog=True,show=True)
-except:
-    plot_image(out_dict["data2D"],out_dict["dataX"],out_dict["dataY"],
-        xtitle='Energy [eV]',ytitle='Theta [mrad]',title='Reflectivity',
-        aspect='auto',show=True)
+if True:
+    from srxraylib.plot.gol import plot, plot_image
+    try:
+        plot(out_dict["data"][0,:],out_dict["data"][-1,:],
+            xtitle=out_dict["labels"][0],ytitle=out_dict["labels"][1],title="xf1f2",
+            xlog=True,ylog=True,show=True)
+    except:
+        plot_image(out_dict["data2D"],out_dict["dataX"],out_dict["dataY"],
+            xtitle='Energy [eV]',ytitle='Theta [mrad]',title='Reflectivity',
+            aspect='auto',show=True)
 #
 # end script
 #
