@@ -9,14 +9,18 @@ from oasys.widgets.exchange import DataExchangeObject
 from xoppylib.power.xoppy_calc_power import xoppy_calc_power
 
 from oasys.widgets.exchange import DataExchangeObject
-from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
+from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget_dabax import XoppyWidgetDabax
 
 import scipy.constants as codata
 
-import xraylib
-from dabax.dabax_xraylib import DabaxXraylib
+try: import xraylib
+except: print("xraylib not available")
 
-class OWxpower(XoppyWidget):
+from dabax.dabax_xraylib import DabaxXraylib
+from dabax.dabax_files import dabax_f1f2_files, dabax_crosssec_files
+
+
+class OWxpower(XoppyWidgetDabax):
     name = "POWER"
     id = "orange.widgets.dataxpower"
     description = "Power Absorbed and Transmitted by Optical Elements"
@@ -74,13 +78,25 @@ class OWxpower(XoppyWidget):
     def __init__(self):
         super().__init__(show_script_tab=True)
 
+    def dabax_show_f1f2(self):
+        return True
+
+    def dabax_show_crosssec(self):
+        return True
+
     def build_gui(self):
 
         self.leftWidgetPart.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding))
         self.leftWidgetPart.setMaximumWidth(self.CONTROL_AREA_WIDTH + 20)
         self.leftWidgetPart.updateGeometry()
 
-        box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-10)
+        # box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-10)
+        ###########
+        tabs_setting = oasysgui.tabWidget(self.controlArea)
+        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)
+        box = oasysgui.createTabPage(tabs_setting, self.name + " Input Parameters")
+        self.tab_dabax = oasysgui.createTabPage(tabs_setting, "Materials Library")
+        ###########
 
         idx = -1 
 
@@ -536,6 +552,15 @@ class OWxpower(XoppyWidget):
 
                     self.process_showers()
                     self.compute()
+                elif exchangeData.get_program_name() == "SRW":
+                    if exchangeData.get_widget_name() =="UNDULATOR_SPECTRUM" :
+                        spectrum = exchangeData.get_content("srw_data")
+
+                        self.input_spectrum = numpy.vstack((spectrum[:, 0], spectrum[:, 1]))
+                        self.input_script = None
+
+                        self.process_showers()
+                        self.compute()
 
         except Exception as exception:
             QMessageBox.critical(self, "Error", str(exception), QMessageBox.Ok)
@@ -694,26 +719,16 @@ class OWxpower(XoppyWidget):
             material_constants_library = xraylib
             material_constants_library_str = "xraylib"
         else:
-            material_constants_library = DabaxXraylib()
-            material_constants_library_str = 'DabaxXraylib()'
+            material_constants_library = DabaxXraylib(file_f1f2=dabax_f1f2_files()[self.DABAX_F1F2_FILE_INDEX],
+                                                      file_CrossSec=dabax_crosssec_files()[self.DABAX_CROSSSEC_FILE_INDEX])
+            material_constants_library_str = 'DabaxXraylib(file_f1f2="%s",file_CrossSec="%s")' % \
+                                             (dabax_f1f2_files()[self.DABAX_F1F2_FILE_INDEX],
+                                              dabax_crosssec_files()[self.DABAX_CROSSSEC_FILE_INDEX])
             print(material_constants_library.info())
 
-        out_dictionary = xoppy_calc_power(
-            energies,
-            source,
-            substance                  = substance,
-            thick                      = thick    ,
-            angle                      = angle    ,
-            dens                       = dens     ,
-            roughness                  = roughness,
-            flags                      = flags    ,
-            nelements                  = self.NELEMENTS + 1,
-            FILE_DUMP                  = self.FILE_DUMP,
-            material_constants_library = material_constants_library,
-                                                )
-
-        print(out_dictionary["info"])
-
+        #
+        # script
+        #
         dict_parameters = {
             "substance"                 : substance_str,
             "thick"                     : thick_str,
@@ -732,6 +747,25 @@ class OWxpower(XoppyWidget):
 
         self.xoppy_script.set_code(script)
 
+        #
+        # run
+        #
+        out_dictionary = xoppy_calc_power(
+            energies,
+            source,
+            substance                  = substance,
+            thick                      = thick    ,
+            angle                      = angle    ,
+            dens                       = dens     ,
+            roughness                  = roughness,
+            flags                      = flags    ,
+            nelements                  = self.NELEMENTS + 1,
+            FILE_DUMP                  = self.FILE_DUMP,
+            material_constants_library = material_constants_library,
+                                                )
+
+        print(out_dictionary["info"])
+
         return  out_dictionary, script
 
     def script_template(self):
@@ -743,7 +777,8 @@ class OWxpower(XoppyWidget):
 
 import numpy
 from xoppylib.power.xoppy_calc_power import xoppy_calc_power
-import xraylib
+try: import xraylib
+except: print("xraylib not available")
 from dabax.dabax_xraylib import DabaxXraylib
 
 out_dictionary = xoppy_calc_power(
@@ -768,12 +803,13 @@ spectral_power = out_dictionary["data"][-1,:]
 #                       
 # example plots
 #
-from srxraylib.plot.gol import plot
-plot(out_dictionary["data"][0,:], out_dictionary["data"][1,:],
-    out_dictionary["data"][0,:], out_dictionary["data"][-1,:],
-    xtitle=out_dictionary["labels"][0],
-    legend=[out_dictionary["labels"][1],out_dictionary["labels"][-1]],
-    title='Spectral Power [W/eV]')
+if True:
+    from srxraylib.plot.gol import plot
+    plot(out_dictionary["data"][0,:], out_dictionary["data"][1,:],
+        out_dictionary["data"][0,:], out_dictionary["data"][-1,:],
+        xtitle=out_dictionary["labels"][0],
+        legend=[out_dictionary["labels"][1],out_dictionary["labels"][-1]],
+        title='Spectral Power [W/eV]')
  
 #
 # end script
