@@ -436,21 +436,9 @@ class OWtc_slit(XoppyWidget):
         # for i in range(self.HARMONICS+1):
         #     if i % 2 != 0: harmonics.append(i)
 
-        K_scan,harmonics,power_array, energy_values_at_flux_peak,flux_values = tuning_curves_on_slit(bl,
-                    Kmin=self.KMIN,
-                    Kmax=self.KMAX,
-                    Kpoints=self.KPOINTS,
-                    harmonics=harmonics,
-                    zero_emittance=zero_emittance,
-                    do_plot_peaks=False,
-                    code=code)
-
         if zero_emittance:
             print("\nNo emittance calculation")
 
-        print("Done")
-
-        # write python script in standard output
         dict_parameters = {
             'ElectronBeamDivergenceH' : self.ELECTRONBEAMDIVERGENCEH,
             'ElectronBeamDivergenceV' : self.ELECTRONBEAMDIVERGENCEV,
@@ -479,8 +467,16 @@ class OWtc_slit(XoppyWidget):
         script = self.script_template().format_map(dict_parameters)
         self.xoppy_script.set_code(script)
 
+        K_scan, harmonics, power_array, energy_values_at_flux_peak, flux_values = tuning_curves_on_slit(bl,
+                    Kmin=self.KMIN,
+                    Kmax=self.KMAX,
+                    Kpoints=self.KPOINTS,
+                    harmonics=harmonics,
+                    zero_emittance=zero_emittance,
+                    do_plot_peaks=False,
+                    code=code)
 
-        return K_scan,harmonics,power_array,energy_values_at_flux_peak,flux_values,script
+        return K_scan, harmonics, power_array, energy_values_at_flux_peak, flux_values, script
 
     def script_template(self):
         return """
@@ -517,9 +513,10 @@ K_scan,harmonics,power_array, energy_values_at_flux_peak,flux_values = tuning_cu
     zero_emittance={zero_emittance},
     do_plot_peaks={do_plot_peaks},
     code="{code}")
-      
-from srxraylib.plot.gol import plot
-plot(energy_values_at_flux_peak,flux_values,xtitle="Photon energy [eV]",ytitle="Flux on slit [photons/s/0.1%bw]")
+
+if True:  
+    from srxraylib.plot.gol import plot
+    plot(energy_values_at_flux_peak,flux_values,xtitle="Photon energy [eV]",ytitle="Flux on slit [photons/s/0.1%bw]")
               
 #
 # end script
