@@ -304,30 +304,6 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
         #     raise Exception("SRW calculation code not supported under Linux")
 
     def do_xoppy_calculation(self):
-        energy, flux, spectral_power, cumulated_power = xoppy_calc_undulator_spectrum(ELECTRONENERGY=self.ELECTRONENERGY,
-                                             ELECTRONENERGYSPREAD=self.ELECTRONENERGYSPREAD,
-                                             ELECTRONCURRENT=self.ELECTRONCURRENT,
-                                             ELECTRONBEAMSIZEH=self.ELECTRONBEAMSIZEH,
-                                             ELECTRONBEAMSIZEV=self.ELECTRONBEAMSIZEV,
-                                             ELECTRONBEAMDIVERGENCEH=self.ELECTRONBEAMDIVERGENCEH,
-                                             ELECTRONBEAMDIVERGENCEV=self.ELECTRONBEAMDIVERGENCEV,
-                                             PERIODID=self.PERIODID,
-                                             NPERIODS=self.NPERIODS,
-                                             KV=self.KV,
-                                             KH=self.KH,
-                                             KPHASE=self.KPHASE,
-                                             DISTANCE=self.DISTANCE,
-                                             GAPH=self.GAPH,
-                                             GAPV=self.GAPV,
-                                             GAPH_CENTER=self.GAPH_CENTER,
-                                             GAPV_CENTER=self.GAPV_CENTER,
-                                             PHOTONENERGYMIN=self.PHOTONENERGYMIN,
-                                             PHOTONENERGYMAX=self.PHOTONENERGYMAX,
-                                             PHOTONENERGYPOINTS=self.PHOTONENERGYPOINTS,
-                                             METHOD=self.METHOD,
-                                             USEEMITTANCES=self.USEEMITTANCES)
-
-        # write python script in standard output
         dict_parameters = {
             "ELECTRONENERGY" : self.ELECTRONENERGY,
             "ELECTRONENERGYSPREAD" : self.ELECTRONENERGYSPREAD,
@@ -358,6 +334,29 @@ class OWundulator_spectrum(XoppyWidget, WidgetDecorator):
 
         self.xoppy_script.set_code(script)
 
+        energy, flux, spectral_power, cumulated_power = xoppy_calc_undulator_spectrum(ELECTRONENERGY=self.ELECTRONENERGY,
+                                             ELECTRONENERGYSPREAD=self.ELECTRONENERGYSPREAD,
+                                             ELECTRONCURRENT=self.ELECTRONCURRENT,
+                                             ELECTRONBEAMSIZEH=self.ELECTRONBEAMSIZEH,
+                                             ELECTRONBEAMSIZEV=self.ELECTRONBEAMSIZEV,
+                                             ELECTRONBEAMDIVERGENCEH=self.ELECTRONBEAMDIVERGENCEH,
+                                             ELECTRONBEAMDIVERGENCEV=self.ELECTRONBEAMDIVERGENCEV,
+                                             PERIODID=self.PERIODID,
+                                             NPERIODS=self.NPERIODS,
+                                             KV=self.KV,
+                                             KH=self.KH,
+                                             KPHASE=self.KPHASE,
+                                             DISTANCE=self.DISTANCE,
+                                             GAPH=self.GAPH,
+                                             GAPV=self.GAPV,
+                                             GAPH_CENTER=self.GAPH_CENTER,
+                                             GAPV_CENTER=self.GAPV_CENTER,
+                                             PHOTONENERGYMIN=self.PHOTONENERGYMIN,
+                                             PHOTONENERGYMAX=self.PHOTONENERGYMAX,
+                                             PHOTONENERGYPOINTS=self.PHOTONENERGYPOINTS,
+                                             METHOD=self.METHOD,
+                                             USEEMITTANCES=self.USEEMITTANCES)
+
         return energy, flux, spectral_power, cumulated_power, script
 
     def script_template(self):
@@ -393,17 +392,18 @@ energy, flux, spectral_power, cumulated_power = xoppy_calc_undulator_spectrum(
 #
 # example plot
 #
-from srxraylib.plot.gol import plot
-
-plot(energy,flux,
-    xtitle="Photon energy [eV]",ytitle="Flux [photons/s/o.1%bw]",title="Undulator Flux",
-    xlog=False,ylog=False,show=False)
-plot(energy,spectral_power,
-    xtitle="Photon energy [eV]",ytitle="Power [W/eV]",title="Undulator Spectral Power",
-    xlog=False,ylog=False,show=False)
-plot(energy,cumulated_power,
-    xtitle="Photon energy [eV]",ytitle="Cumulated Power [W]",title="Undulator Cumulated Power",
-    xlog=False,ylog=False,show=True)
+if True:
+    from srxraylib.plot.gol import plot
+    
+    plot(energy,flux,
+        xtitle="Photon energy [eV]",ytitle="Flux [photons/s/o.1%bw]",title="Undulator Flux",
+        xlog=False,ylog=False,show=False)
+    plot(energy,spectral_power,
+        xtitle="Photon energy [eV]",ytitle="Power [W/eV]",title="Undulator Spectral Power",
+        xlog=False,ylog=False,show=False)
+    plot(energy,cumulated_power,
+        xtitle="Photon energy [eV]",ytitle="Cumulated Power [W]",title="Undulator Cumulated Power",
+        xlog=False,ylog=False,show=True)
 #
 # end script
 #
